import * as utilities from '/base-standard/maps/map-utilities.js';
export function generateResources(iWidth, iHeight, iNumWest, iNumEast, mapInfo) {
    let bEastBias = false;
    if (iNumEast > iNumWest) {
        console.log("EastSide");
        bEastBias = true;
    }
    let resourceHemisphere = new Array(GameInfo.Resources.length);
    let resourceRegionalCount = new Array(2);
    resourceRegionalCount[0] = 0;
    resourceRegionalCount[1] = 0;
    let resourceWeight = new Array(GameInfo.Resources.length);
    let resourceRunningWeight = new Array(GameInfo.Resources.length);
    let importantResourceRegionalCountHome = new Array(GameInfo.Resources.length);
    let importantResourceRegionalCountDistant = new Array(GameInfo.Resources.length);
    let resourcesPlacedCount = new Array(GameInfo.Resources.length);
    //Initial Resource data
    for (var resourceIdx = 0; resourceIdx < GameInfo.Resources.length; resourceIdx++) {
        resourceHemisphere[resourceIdx] = 0;
        resourceWeight[resourceIdx] = 0;
        resourceRunningWeight[resourceIdx] = 0;
        resourcesPlacedCount[resourceIdx] = 0;
    }
    // Find all resources
    let aResourceTypes = [];
    let resources = ResourceBuilder.getGeneratedMapResources();
    for (let ridx = 0; ridx < resources.length; ++ridx) {
        var resourceInfo = GameInfo.Resources.lookup(resources[ridx]);
        if (resourceInfo && resourceInfo.Tradeable) {
            resourceWeight[resourceInfo.$index] = resourceInfo.Weight;
            resourceHemisphere[resourceInfo.$index] = resourceInfo.Hemispheres;
            aResourceTypes.push(resourceInfo.$index);
            importantResourceRegionalCountHome[resourceInfo.$index] = 0;
            importantResourceRegionalCountDistant[resourceInfo.$index] = 0;
        }
    }
    //Generate Poisson Map
    let seed = GameplayMap.getRandomSeed();
    let avgDistanceBetweenPoints = 3;
    let normalizedRangeSmoothing = 2;
    let poisson = TerrainBuilder.generatePoissonMap(seed, avgDistanceBetweenPoints, normalizedRangeSmoothing);
    for (let iY = iHeight - 1; iY >= 0; iY--) {
        for (let iX = 0; iX < iWidth; iX++) {
            let hemisphere = GameplayMap.getHemisphere(iX);
            let index = iY * iWidth + iX;
            if (poisson[index] >= 1) {
                //Generate a list of valid resources at this plot
                let resources = [];
                aResourceTypes.forEach(resourceIdx => {
                    let iBuffer = Math.floor(iWidth / 28.0);
                    let assignedHempisphere = ResourceBuilder.getResourceHemisphere(resourceIdx);
                    let continent = mapInfo[iX + iY * iWidth].continent;
                    if (assignedHempisphere > -1) {
                        if (assignedHempisphere == 2 || assignedHempisphere == hemisphere) {
                            if (resourceHemisphere[resourceIdx] == 0) {
                                if ((continent == 0 && bEastBias == false) || (continent == 2 && bEastBias == true)) {
                                    if (canHaveFlowerPlot(iX, iY, resourceIdx)) {
                                        resources.push(resourceIdx);
                                        //console.log("Foreign Hemishpere: " + iX + ", " + iY);
                                    }
                                }
                            }
                            else if (resourceHemisphere[resourceIdx] == 1) {
                                if (continent == 1 || (continent == 0 && bEastBias) || (continent == 2 && !bEastBias)) {
                                    if (canHaveFlowerPlot(iX, iY, resourceIdx)) {
                                        resources.push(resourceIdx);
                                        //console.log("Foreign: " + iX + ", " + iY);
                                    }
                                }
                            }
                            else if (resourceHemisphere[resourceIdx] == 2 || resourceHemisphere[resourceIdx] == -1) {
                                if (canHaveFlowerPlot(iX, iY, resourceIdx)) {
                                    resources.push(resourceIdx);
                                }
                            }
                        }
                    }
                });
                //Select the heighest weighted (ties are a coin flip) resource
                if (resources.length > 0) {
                    let resourceChosen = ResourceTypes.NO_RESOURCE;
                    let resourceChosenIndex = 0;
                    for (let iI = 0; iI < resources.length; iI++) {
                        if (resourceChosen == ResourceTypes.NO_RESOURCE) {
                            resourceChosen = resources[iI];
                            resourceChosenIndex = resources[iI];
                        }
                        else {
                            if (resourceRunningWeight[resources[iI]] > resourceRunningWeight[resourceChosenIndex]) {
                                resourceChosen = resources[iI];
                                resourceChosenIndex = resources[iI];
                            }
                            else if (resourceRunningWeight[resources[iI]] == resourceRunningWeight[resourceChosenIndex]) {
                                let iRoll = TerrainBuilder.getRandomNumber(2, "Resource Scatter");
                                if (iRoll >= 1) {
                                    resourceChosen = resources[iI];
                                    resourceChosenIndex = resources[iI];
                                }
                            }
                        }
                    }
                    //Place the selected resource
                    if (resourceChosen != ResourceTypes.NO_RESOURCE) {
                        let iResourcePlotIndex = getFlowerPlot(iX, iY, resourceChosen);
                        if (iResourcePlotIndex != -1) {
                            let iLocation = GameplayMap.getLocationFromIndex(iResourcePlotIndex);
                            let iResourceX = iLocation.x;
                            let iResourceY = iLocation.y;
                            ResourceBuilder.setResourceType(iResourceX, iResourceY, resourceChosen);
                            resourceRunningWeight[resourceChosenIndex] -= resourceWeight[resourceChosenIndex];
                            resourcesPlacedCount[resourceChosenIndex]++;
                            GameplayMap.getHemisphere(iX) == 0 ? importantResourceRegionalCountHome[resourceChosenIndex]++ : importantResourceRegionalCountDistant[resourceChosenIndex]++;
                        }
                        else {
                            console.log("Resource Index Failure");
                        }
                    }
                    else {
                        console.log("Resource Type Failure");
                    }
                }
            }
        }
    }
    //Some resources are required for gameplay for the age, if not enough were generated, place them on the map now. Some differences from transition placement but might want to look at making a utility function for these placements.
    let checkHomeHemisphereLP = true;
    let checkDistantHemisphereLP = true;
    for (let iY = 0; iY < iHeight; iY++) {
        for (let iX = 0; iX < iWidth; iX++) {
            let hemisphere = GameplayMap.getHemisphere(iX);
            let resourceAtLocation = GameplayMap.getResourceType(iX, iY);
            if (resourceAtLocation == ResourceTypes.NO_RESOURCE) {
                let resourcesEligible = [];
                for (let i = 0; i < resourcesPlacedCount.length; ++i) {
                    let resourceToPlace = GameInfo.Resources.lookup(i);
                    if (resourceToPlace) {
                        let assignedHempisphere = ResourceBuilder.getResourceHemisphere(i);
                        if (assignedHempisphere > -1) {
                            if (assignedHempisphere == 2 || assignedHempisphere == hemisphere) {
                                if (hemisphere == 0 && importantResourceRegionalCountHome[i] < resourceToPlace.MinimumPerHemisphere || hemisphere == 1 && importantResourceRegionalCountDistant[i] < resourceToPlace.MinimumPerHemisphere) {
                                    //Once LP class checks are complete, no need to do them anymore
                                    if (checkHomeHemisphereLP && hemisphere == 0) {
                                        checkHomeHemisphereLP = ResourceBuilder.isResourceClassRequiredForLegacyPath(i, hemisphere);
                                    }
                                    else if (checkDistantHemisphereLP && hemisphere == 1) {
                                        checkDistantHemisphereLP = ResourceBuilder.isResourceClassRequiredForLegacyPath(i, hemisphere);
                                    }
                                    if (ResourceBuilder.isResourceRequiredForAge(i, hemisphere)) {
                                        if (ResourceBuilder.canHaveResource(iX, iY, i)) {
                                            //Try not to place adjacent to other resources. The new Poission map causes the old one to be ignored, which causes clumping.
                                            let hasAdjResource = false;
                                            for (let iDirection = 0; iDirection < DirectionTypes.NUM_DIRECTION_TYPES; iDirection++) {
                                                let iIndex = GameplayMap.getIndexFromXY(iX, iY);
                                                let iLocation = GameplayMap.getLocationFromIndex(iIndex);
                                                let iAdjacentX = GameplayMap.getAdjacentPlotLocation(iLocation, iDirection).x;
                                                let iAdjacentY = GameplayMap.getAdjacentPlotLocation(iLocation, iDirection).y;
                                                if (GameplayMap.getResourceType(iAdjacentX, iAdjacentY) != ResourceTypes.NO_RESOURCE) {
                                                    hasAdjResource = true;
                                                    break;
                                                }
                                            }
                                            if (!hasAdjResource) {
                                                resourcesEligible.push(i);
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
                //Select the heighest weighted (ties are a coin flip) resource
                let resourceChosenIndex = -1;
                if (resourcesEligible.length > 0) {
                    let resourceChosen = ResourceTypes.NO_RESOURCE;
                    for (let iI = 0; iI < resourcesEligible.length; iI++) {
                        if (resourceChosen == ResourceTypes.NO_RESOURCE) {
                            resourceChosen = resourcesEligible[iI];
                            resourceChosenIndex = resourcesEligible[iI];
                        }
                        else {
                            if (resourceRunningWeight[resourcesEligible[iI]] > resourceRunningWeight[resourceChosenIndex]) {
                                resourceChosen = resourcesEligible[iI];
                                resourceChosenIndex = resourcesEligible[iI];
                            }
                            else if (resourceRunningWeight[resourcesEligible[iI]] == resourceRunningWeight[resourceChosenIndex]) {
                                let iRoll = TerrainBuilder.getRandomNumber(2, "Resource Scatter");
                                if (iRoll >= 1) {
                                    resourceChosen = resourcesEligible[iI];
                                    resourceChosenIndex = resourcesEligible[iI];
                                }
                            }
                        }
                    }
                }
                if (resourceChosenIndex > -1) {
                    ResourceBuilder.setResourceType(iX, iY, resourceChosenIndex);
                    resourceRunningWeight[resourceChosenIndex] -= resourceWeight[resourceChosenIndex];
                    let name = GameInfo.Resources.lookup(resourceChosenIndex)?.Name;
                    console.log("Force Placed " + Locale.compose(name) + " at (" + iX + ", " + iY + ")");
                    hemisphere == 0 ? importantResourceRegionalCountHome[resourceChosenIndex]++ : importantResourceRegionalCountDistant[resourceChosenIndex]++;
                    break;
                }
            }
        }
    }
}
//Can I have a resource in this flower?
export function canHaveFlowerPlot(iX, iY, resourceType) {
    if (ResourceBuilder.canHaveResource(iX, iY, resourceType)) {
        return true;
    }
    for (let iDirection = 0; iDirection < DirectionTypes.NUM_DIRECTION_TYPES; iDirection++) {
        let iIndex = GameplayMap.getIndexFromXY(iX, iY);
        let iLocation = GameplayMap.getLocationFromIndex(iIndex);
        let iAdjacentX = GameplayMap.getAdjacentPlotLocation(iLocation, iDirection).x;
        let iAdjacentY = GameplayMap.getAdjacentPlotLocation(iLocation, iDirection).y;
        if (ResourceBuilder.canHaveResource(iAdjacentX, iAdjacentY, resourceType)) {
            return true;
        }
    }
    return false;
}
//Return a plot index for this resource
//First choosing the initial plot, otherwise it randomly chooses a valid plot from the sorounding ring
export function getFlowerPlot(iX, iY, resourceType) {
    if (ResourceBuilder.canHaveResource(iX, iY, resourceType)) {
        return GameplayMap.getIndexFromXY(iX, iY);
    }
    let resourcePlotIndexes = [];
    for (let iDirection = 0; iDirection < DirectionTypes.NUM_DIRECTION_TYPES; iDirection++) {
        let iIndex = GameplayMap.getIndexFromXY(iX, iY);
        let iLocation = GameplayMap.getLocationFromIndex(iIndex);
        let iAdjacentX = GameplayMap.getAdjacentPlotLocation(iLocation, iDirection).x;
        let iAdjacentY = GameplayMap.getAdjacentPlotLocation(iLocation, iDirection).y;
        let iAdjacentIndex = GameplayMap.getIndexFromXY(iAdjacentX, iAdjacentY);
        if (ResourceBuilder.canHaveResource(iAdjacentX, iAdjacentY, resourceType)) {
            resourcePlotIndexes.push(iAdjacentIndex);
        }
    }
    if (resourcePlotIndexes.length > 0) {
        return utilities.shuffle(resourcePlotIndexes)[0];
    }
    else {
        return -1;
    }
}

//# sourceMappingURL=file:///base-standard/maps/resource-generator.js.map
